@@ -483,13 +483,13 @@ class DataConstantsTransformerForm(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
-
     index = forms.IntegerField(required=False,
                                widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                                initial=0)
     value = forms.CharField(required=False,
                             widget=forms.TextInput(attrs={'class': 'form-control'}))
-    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
+                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
         model = DataTransformerConstants
@@ -502,7 +502,8 @@ class HardwareTransformerForm(forms.ModelForm):
     index = forms.IntegerField(required=False,
                                widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                                initial=0)
-    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
+                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
         model = HardwareDataTransformer
@@ -513,24 +514,49 @@ class HardwareStatsTransformerForm(forms.ModelForm):
     id = forms.CharField(required=False,
                          widget=forms.HiddenInput(attrs={'class': 'form-control'}))
     index = forms.IntegerField(required=False,
-                            widget=forms.HiddenInput(attrs={'class': 'form-control'}),
+                               widget=forms.HiddenInput(attrs={'class': 'form-control'}),
                                initial=0)
-    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(), widget=forms.HiddenInput(attrs={'class': 'form-control'}))
-
+    data_transformer = forms.ModelChoiceField(required=False, queryset=DataTransformer.objects.all(),
+                                              widget=forms.HiddenInput(attrs={'class': 'form-control'}))
 
     class Meta:
         model = HardwareStatsDataTransformer
         fields = ('__all__')
 
 
-DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, extra=1,
+class BaseInlineFormSetWithAddMoreButton(BaseInlineFormSet):
+    def as_p(self):
+        button_id = str(self.prefix) + 'add_more'
+        empty_form_id = str(self.prefix) + 'empty_form'
+        empty_form_as_p = str(self.empty_form.as_p())
+        div_name = str(self.prefix) + 'div'
+
+        forms_as_p = super(BaseInlineFormSetWithAddMoreButton, self).as_p()
+        p = f"""<div class="row" id="{div_name}">
+        {forms_as_p}
+        </div>
+        <input type="button" value="Add More" id="{button_id}">
+                    <div id="{empty_form_id}" style="display:none">
+                    {empty_form_as_p}
+                    </div>
+                    
+        <script>
+    $('#{button_id}').click(function() {{
+        var form_idx = $('#{self.prefix}-TOTAL_FORMS').val();
+        $('#{div_name}').append($('#{empty_form_id}').html().replace(/__prefix__/g, form_idx));
+        $('#{self.prefix}-TOTAL_FORMS').val(parseInt(form_idx) + 1);
+    }});
+</script>"""
+        return p
+
+
+DataConstantsTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformerConstants, formset=BaseInlineFormSetWithAddMoreButton, extra=0,
                                                               form=DataConstantsTransformerForm, can_delete_extra=True)
 # HardwareTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareDataTransformer, extra=1,
 #                                                          form=HardwareTransformerForm, can_delete_extra=True)
-HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=1,
+HardwareStatsTransformerInlineFormset = inlineformset_factory(DataTransformer, HardwareStatsDataTransformer, extra=0, formset=BaseInlineFormSetWithAddMoreButton,
                                                               form=HardwareStatsTransformerForm, can_delete_extra=True)
 
-
 from django.utils.safestring import mark_safe
 
 
@@ -555,13 +581,13 @@ class DataTransform(forms.ModelForm):
             if self.instance.get_children().count() >= 0:
                 print('Adding to child form node: ' + str(self.instance.get_children().count() + 1))
 
-
                 self.nested_data_transformer_form = dataTransformerInlineFormset(
                     instance=self.instance,
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id), 'id':str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
+                              'id': str(uuid.uuid4())}],
                     prefix=str(self.instance.id) + 'dataTransformerInlineFormset'
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count() + 1
@@ -582,7 +608,7 @@ class DataTransform(forms.ModelForm):
             queryset=self.instance.datatransformerconstants_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'DataConstantsTransformerInlineFormset',
+            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
@@ -593,7 +619,7 @@ class DataTransform(forms.ModelForm):
             instance=hardware_outputs,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'HardwareTransformerForm',
+            prefix=str(self.instance.id) + 'HardwareTransformerForm',
             initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
         )
 
@@ -602,13 +628,12 @@ class DataTransform(forms.ModelForm):
             queryset=self.instance.hardwarestatsdatatransformer_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'HardwareStatsTransformerInlineFormset',
+            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
         self.nested_hardware_stats_form.extra = self.instance.hardwarestatsdatatransformer_set.count() + 1
 
-
     def as_p(self):
         print('wtf')
         print('instance: ' + str(self.instance.id))
@@ -674,10 +699,11 @@ class DataTransform(forms.ModelForm):
 
 # DataTransformerFormset = form_factory(DataTransformer,formset=DataTransformerForm)
 dataTransformerInlineFormset = inlineformset_factory(DataTransformer, DataTransformer,
-                                                                     fk_name='parent',
-                                                                     can_delete_extra=True,
-                                                                     form=DataTransform
-                                                                     )
+                                                     fk_name='parent',
+                                                     can_delete_extra=True,
+                                                     form=DataTransform
+                                                     )
+
 
 class DataTransformRoot(forms.ModelForm):
     id = forms.CharField(required=False,
@@ -702,7 +728,8 @@ class DataTransformRoot(forms.ModelForm):
                     queryset=self.instance.get_children(),
                     data=self.data if self.is_bound else None,
                     files=self.files if self.is_bound else None,
-                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id), 'id':str(uuid.uuid4())}],
+                    initial=[{'parent': str(self.instance.pk), 'accessory': str(self.instance.accessory.id),
+                              'id': str(uuid.uuid4())}],
                     prefix=str(self.instance.id) + 'dataTransformerInlineFormset',
                 )
                 self.nested_data_transformer_form.extra = self.instance.get_children().count() + 1
@@ -724,19 +751,18 @@ class DataTransformRoot(forms.ModelForm):
             queryset=self.instance.datatransformerconstants_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'DataConstantsTransformerInlineFormset',
+            prefix=str(self.instance.id) + 'DataConstantsTransformerInlineFormset',
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
         self.nested_constants_form.extra = self.instance.datatransformerconstants_set.count() + 1
 
-
         hardware_outputs = self.instance.hardwaredatatransformer_set.first()
         self.nested_hardware_outputs_form = HardwareTransformerForm(
             instance=hardware_outputs,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'HardwareTransformerForm',
+            prefix=str(self.instance.id) + 'HardwareTransformerForm',
             initial={'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}
         )
 
@@ -745,7 +771,7 @@ class DataTransformRoot(forms.ModelForm):
             queryset=self.instance.hardwarestatsdatatransformer_set,
             data=self.data if self.is_bound else None,
             files=self.files if self.is_bound else None,
-            prefix=str(self.instance.id)+'HardwareStatsTransformerInlineFormset',
+            prefix=str(self.instance.id) + 'HardwareStatsTransformerInlineFormset',
             initial=[{'data_transformer': str(self.instance.pk), 'id': str(uuid.uuid4())}]
 
         )
